@@ -40,8 +40,6 @@ const gchar	*gedit_dirs_get_user_styles_dir		(void);
 
 const gchar	*gedit_dirs_get_user_plugins_dir	(void);
 
-const gchar	*gedit_dirs_get_gedit_data_dir		(void);
-
 const gchar	*gedit_dirs_get_gedit_locale_dir	(void);
 
 const gchar	*gedit_dirs_get_gedit_lib_dir		(void);
@@ -50,8 +48,6 @@ const gchar	*gedit_dirs_get_gedit_plugins_dir	(void);
 
 const gchar	*gedit_dirs_get_gedit_plugins_data_dir	(void);
 
-gchar		*gedit_dirs_get_ui_file			(const gchar *file);
-
 G_END_DECLS
 
 #endif /* __GEDIT_DIRS_H__ */
