@@ -31,7 +31,6 @@
 #include "gedit-multi-notebook.h"
 #include "gedit-notebook.h"
 #include "gedit-notebook-popup-menu.h"
-#include "gedit-small-button.h"
 #include "gedit-tab.h"
 #include "gedit-tab-private.h"
 #include "gedit-utils.h"
@@ -1537,6 +1536,11 @@ row_on_button_pressed (GtkWidget      *row_event_box,
 			GeditTab *tab = GEDIT_TAB (document_row->ref);
 			GtkWidget *menu = gedit_notebook_popup_menu_new (window, tab);
 
+			g_signal_connect (menu,
+					  "selection-done",
+					  G_CALLBACK (gtk_widget_destroy),
+					  NULL);
+
 			gtk_menu_popup_for_device (GTK_MENU (menu),
 			                           gdk_event_get_device ((GdkEvent *)event),
 			                           NULL, NULL,
@@ -1582,6 +1586,9 @@ row_create (GtkWidget *row)
 {
 	GeditDocumentsGenericRow *generic_row = (GeditDocumentsGenericRow *)row;
 	GtkWidget *event_box;
+	GtkStyleContext *context;
+	GtkWidget *image;
+	GIcon *icon;
 
 	gedit_debug (DEBUG_PANEL);
 
@@ -1599,7 +1606,21 @@ row_create (GtkWidget *row)
 	gtk_widget_set_halign (generic_row->status_label, GTK_ALIGN_END);
 	gtk_widget_set_valign (generic_row->status_label, GTK_ALIGN_CENTER);
 
-	generic_row->close_button = gedit_close_button_new ();
+	generic_row->close_button = GTK_WIDGET (g_object_new (GTK_TYPE_BUTTON,
+	                                                      "relief", GTK_RELIEF_NONE,
+	                                                      "focus-on-click", FALSE,
+	                                                      NULL));
+
+	context = gtk_widget_get_style_context (generic_row->close_button);
+	gtk_style_context_add_class (context, "flat");
+	gtk_style_context_add_class (context, "small-button");
+
+	icon = g_themed_icon_new_with_default_fallbacks ("window-close-symbolic");
+	image = gtk_image_new_from_gicon (icon, GTK_ICON_SIZE_MENU);
+	gtk_widget_show (image);
+	g_object_unref (icon);
+
+	gtk_container_add (GTK_CONTAINER (generic_row->close_button), image);
 
 	gtk_box_pack_start (GTK_BOX (generic_row->box),
 	                    generic_row->label, FALSE, FALSE, 0);
