@@ -2,7 +2,7 @@
  * gedit-spell-plugin.c
  *
  * Copyright (C) 2002-2005 Paolo Maggi
- * Copyright (C) 2015 Sébastien Wilmet
+ * Copyright (C) 2015-2016 Sébastien Wilmet
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,11 +22,9 @@
 
 #include <glib/gi18n.h>
 #include <gedit/gedit-debug.h>
-#include <gedit/gedit-statusbar.h>
 #include <gedit/gedit-app.h>
 #include <gedit/gedit-window.h>
 #include <gedit/gedit-window-activatable.h>
-#include <gtksourceview/gtksource.h>
 #include <gspell/gspell.h>
 
 #include "gedit-spell-app-activatable.h"
@@ -41,31 +39,11 @@
 
 #define SPELL_ENABLED_STR "1"
 
-#define VIEW_DATA_KEY "GeditSpellPlugin-ViewData"
-
 static void gedit_window_activatable_iface_init (GeditWindowActivatableInterface *iface);
 
 struct _GeditSpellPluginPrivate
 {
-	GeditWindow    *window;
-
-	guint           statusbar_context_id;
-	gulong          tab_added_id;
-	gulong          tab_removed_id;
-};
-
-typedef struct _ViewData ViewData;
-
-struct _ViewData
-{
-	GeditSpellPlugin *plugin;
-	GeditView *view;
-	GspellInlineCheckerGtv *inline_checker;
-
-	/* The doc is also needed, to be sure that the signals are disconnected
-	 * on the same doc.
-	 */
-	GeditDocument *doc;
+	GeditWindow *window;
 };
 
 enum
@@ -82,102 +60,6 @@ G_DEFINE_DYNAMIC_TYPE_EXTENDED (GeditSpellPlugin,
 							       gedit_window_activatable_iface_init)
 				G_ADD_PRIVATE_DYNAMIC (GeditSpellPlugin))
 
-static void	spell_cb			(GSimpleAction *action, GVariant *parameter, gpointer data);
-static void	set_language_cb			(GSimpleAction *action, GVariant *parameter, gpointer data);
-static void	inline_checker_activate_cb	(GSimpleAction *action, GVariant *parameter, gpointer data);
-static void	inline_checker_change_state_cb	(GSimpleAction *action, GVariant *state, gpointer data);
-
-static void	on_document_loaded		(GeditDocument *doc, ViewData *data);
-static void	on_document_saved		(GeditDocument *doc, ViewData *data);
-static void	set_inline_checker_from_metadata (ViewData *data);
-
-static GActionEntry action_entries[] =
-{
-	{ "check-spell", spell_cb },
-	{ "config-spell", set_language_cb },
-	{ "inline-checker", inline_checker_activate_cb, NULL, "false", inline_checker_change_state_cb }
-};
-
-static GQuark spell_checker_id = 0;
-
-static ViewData *
-view_data_new (GeditSpellPlugin *plugin,
-	       GeditView        *view)
-{
-	ViewData *data;
-
-	data = g_slice_new (ViewData);
-	data->plugin = g_object_ref (plugin);
-	data->view = g_object_ref (view);
-	data->inline_checker = NULL;
-
-	data->doc = GEDIT_DOCUMENT (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
-	g_object_ref (data->doc);
-
-	g_signal_connect (data->doc,
-			  "loaded",
-			  G_CALLBACK (on_document_loaded),
-			  data);
-
-	g_signal_connect (data->doc,
-			  "saved",
-			  G_CALLBACK (on_document_saved),
-			  data);
-
-	set_inline_checker_from_metadata (data);
-
-	return data;
-}
-
-static void
-view_data_free (ViewData *data)
-{
-	if (data == NULL)
-	{
-		return;
-	}
-
-	if (data->doc != NULL)
-	{
-		g_signal_handlers_disconnect_by_func (data->doc, on_document_loaded, data);
-		g_signal_handlers_disconnect_by_func (data->doc, on_document_saved, data);
-
-		g_object_unref (data->doc);
-	}
-
-	if (data->inline_checker != NULL && data->view != NULL)
-	{
-		gspell_inline_checker_gtv_detach_view (data->inline_checker,
-						       GTK_TEXT_VIEW (data->view));
-	}
-
-	g_clear_object (&data->plugin);
-	g_clear_object (&data->view);
-	g_clear_object (&data->inline_checker);
-
-	g_slice_free (ViewData, data);
-}
-
-static void
-gedit_spell_plugin_init (GeditSpellPlugin *plugin)
-{
-	gedit_debug_message (DEBUG_PLUGINS, "GeditSpellPlugin initializing");
-
-	plugin->priv = gedit_spell_plugin_get_instance_private (plugin);
-}
-
-static void
-gedit_spell_plugin_dispose (GObject *object)
-{
-	GeditSpellPlugin *plugin = GEDIT_SPELL_PLUGIN (object);
-
-	gedit_debug_message (DEBUG_PLUGINS, "GeditSpellPlugin disposing");
-
-	g_clear_object (&plugin->priv->window);
-
-	G_OBJECT_CLASS (gedit_spell_plugin_parent_class)->dispose (object);
-}
-
 static void
 gedit_spell_plugin_set_property (GObject      *object,
 				 guint         prop_id,
@@ -219,24 +101,40 @@ gedit_spell_plugin_get_property (GObject    *object,
 }
 
 static void
-language_notify_cb (GspellChecker *checker,
-		    GParamSpec    *pspec,
-		    GeditDocument *doc)
+gedit_spell_plugin_dispose (GObject *object)
 {
-	const GspellLanguage *lang;
-	const gchar *language_code;
+	GeditSpellPlugin *plugin = GEDIT_SPELL_PLUGIN (object);
 
-	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
+	gedit_debug_message (DEBUG_PLUGINS, "GeditSpellPlugin disposing");
 
-	lang = gspell_checker_get_language (checker);
-	g_return_if_fail (lang != NULL);
+	g_clear_object (&plugin->priv->window);
 
-	language_code = gspell_language_get_code (lang);
-	g_return_if_fail (language_code != NULL);
+	G_OBJECT_CLASS (gedit_spell_plugin_parent_class)->dispose (object);
+}
 
-	gedit_document_set_metadata (doc,
-				     GEDIT_METADATA_ATTRIBUTE_SPELL_LANGUAGE, language_code,
-				     NULL);
+static void
+gedit_spell_plugin_class_init (GeditSpellPluginClass *klass)
+{
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+	object_class->set_property = gedit_spell_plugin_set_property;
+	object_class->get_property = gedit_spell_plugin_get_property;
+	object_class->dispose = gedit_spell_plugin_dispose;
+
+	g_object_class_override_property (object_class, PROP_WINDOW, "window");
+}
+
+static void
+gedit_spell_plugin_class_finalize (GeditSpellPluginClass *klass)
+{
+}
+
+static void
+gedit_spell_plugin_init (GeditSpellPlugin *plugin)
+{
+	gedit_debug_message (DEBUG_PLUGINS, "GeditSpellPlugin initializing");
+
+	plugin->priv = gedit_spell_plugin_get_instance_private (plugin);
 }
 
 static const GspellLanguage *
@@ -256,42 +154,30 @@ get_language_from_metadata (GeditDocument *doc)
 	return lang;
 }
 
-static GspellChecker *
-get_spell_checker_from_document (GeditDocument *doc)
+static void
+check_spell_cb (GSimpleAction *action,
+		GVariant      *parameter,
+		gpointer       data)
 {
-	GspellChecker *checker;
-	gpointer data;
+	GeditSpellPlugin *plugin = GEDIT_SPELL_PLUGIN (data);
+	GeditSpellPluginPrivate *priv;
+	GeditView *view;
+	GspellNavigator *navigator;
+	GtkWidget *dialog;
 
 	gedit_debug (DEBUG_PLUGINS);
 
-	g_return_val_if_fail (doc != NULL, NULL);
+	priv = plugin->priv;
 
-	data = g_object_get_qdata (G_OBJECT (doc), spell_checker_id);
+	view = gedit_window_get_active_view (priv->window);
+	g_return_if_fail (view != NULL);
 
-	if (data == NULL)
-	{
-		const GspellLanguage *lang;
+	navigator = gspell_navigator_text_new (GTK_TEXT_VIEW (view));
+	dialog = gspell_checker_dialog_new (GTK_WINDOW (priv->window), navigator);
+	g_object_unref (navigator);
 
-		lang = get_language_from_metadata (doc);
-		checker = gspell_checker_new (lang);
-
-		g_object_set_qdata_full (G_OBJECT (doc),
-					 spell_checker_id,
-					 checker,
-					 g_object_unref);
-
-		g_signal_connect (checker,
-				  "notify::language",
-				  G_CALLBACK (language_notify_cb),
-				  doc);
-	}
-	else
-	{
-		g_return_val_if_fail (GSPELL_IS_CHECKER (data), NULL);
-		checker = data;
-	}
-
-	return checker;
+	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+	gtk_widget_show (dialog);
 }
 
 static void
@@ -322,7 +208,7 @@ set_language_cb (GSimpleAction *action,
 	GspellChecker *checker;
 	const GspellLanguage *lang;
 	GtkWidget *dialog;
-	GtkWindowGroup *wg;
+	GtkWindowGroup *window_group;
 
 	gedit_debug (DEBUG_PLUGINS);
 
@@ -331,7 +217,7 @@ set_language_cb (GSimpleAction *action,
 	doc = gedit_window_get_active_document (priv->window);
 	g_return_if_fail (doc != NULL);
 
-	checker = get_spell_checker_from_document (doc);
+	checker = gspell_text_buffer_get_spell_checker (GTK_TEXT_BUFFER (doc));
 	g_return_if_fail (checker != NULL);
 
 	lang = gspell_checker_get_language (checker);
@@ -345,9 +231,9 @@ set_language_cb (GSimpleAction *action,
 				checker, "language",
 				G_BINDING_DEFAULT);
 
-	wg = gedit_window_get_group (priv->window);
+	window_group = gedit_window_get_group (priv->window);
 
-	gtk_window_group_add_window (wg, GTK_WINDOW (dialog));
+	gtk_window_group_add_window (window_group, GTK_WINDOW (dialog));
 
 	gtk_dialog_add_button (GTK_DIALOG (dialog),
 			       _("_Help"),
@@ -361,72 +247,6 @@ set_language_cb (GSimpleAction *action,
 	gtk_widget_show (dialog);
 }
 
-static void
-spell_cb (GSimpleAction *action,
-	  GVariant      *parameter,
-	  gpointer       data)
-{
-	GeditSpellPlugin *plugin = GEDIT_SPELL_PLUGIN (data);
-	GeditSpellPluginPrivate *priv;
-	GeditView *view;
-	GeditDocument *doc;
-	GspellChecker *checker;
-	GspellNavigator *navigator;
-	GtkWidget *dialog;
-
-	gedit_debug (DEBUG_PLUGINS);
-
-	priv = plugin->priv;
-
-	view = gedit_window_get_active_view (priv->window);
-	g_return_if_fail (view != NULL);
-
-	doc = GEDIT_DOCUMENT (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
-
-	checker = get_spell_checker_from_document (doc);
-	g_return_if_fail (checker != NULL);
-
-	navigator = gspell_navigator_gtv_new (GTK_TEXT_VIEW (view), checker);
-	dialog = gspell_checker_dialog_new (GTK_WINDOW (priv->window), navigator);
-	g_object_unref (navigator);
-
-	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
-	gtk_widget_show (dialog);
-}
-
-static void
-set_inline_checker (ViewData *data,
-		    gboolean  active)
-{
-	if (!active)
-	{
-		if (data->inline_checker != NULL && data->view != NULL)
-		{
-			gspell_inline_checker_gtv_detach_view (data->inline_checker,
-							       GTK_TEXT_VIEW (data->view));
-		}
-
-		g_clear_object (&data->inline_checker);
-	}
-	else if (data->inline_checker == NULL)
-	{
-		GspellChecker *checker;
-		GtkTextBuffer *buffer;
-
-		checker = get_spell_checker_from_document (data->doc);
-		g_return_if_fail (checker != NULL);
-
-		buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (data->view));
-		g_return_if_fail (buffer == GTK_TEXT_BUFFER (data->doc));
-
-		data->inline_checker = gspell_inline_checker_gtv_new (GTK_TEXT_BUFFER (data->doc),
-								      checker);
-
-		gspell_inline_checker_gtv_attach_view (data->inline_checker,
-						       GTK_TEXT_VIEW (data->view));
-	}
-}
-
 static void
 inline_checker_activate_cb (GSimpleAction *action,
 			    GVariant      *parameter,
@@ -489,12 +309,10 @@ inline_checker_change_state_cb (GSimpleAction *action,
 	view = gedit_window_get_active_view (priv->window);
 	if (view != NULL)
 	{
-		ViewData *data = g_object_get_data (G_OBJECT (view), VIEW_DATA_KEY);
+		GspellInlineCheckerText *inline_checker;
 
-		if (data != NULL)
-		{
-			set_inline_checker (data, active);
-		}
+		inline_checker = gspell_text_view_get_inline_checker (GTK_TEXT_VIEW (view));
+		gspell_inline_checker_text_set_enabled (inline_checker, active);
 
 		g_simple_action_set_state (action, g_variant_new_boolean (active));
 	}
@@ -504,7 +322,9 @@ static void
 update_ui (GeditSpellPlugin *plugin)
 {
 	GeditSpellPluginPrivate *priv;
-	GeditView *view;
+	GeditTab *tab;
+	GeditView *view = NULL;
+	gboolean editable_view;
 	GAction *check_spell_action;
 	GAction *config_spell_action;
 	GAction *inline_checker_action;
@@ -513,93 +333,111 @@ update_ui (GeditSpellPlugin *plugin)
 
 	priv = plugin->priv;
 
-	view = gedit_window_get_active_view (priv->window);
+	tab = gedit_window_get_active_tab (priv->window);
+	if (tab != NULL)
+	{
+		view = gedit_tab_get_view (tab);
+	}
+
+	editable_view = (view != NULL) && gtk_text_view_get_editable (GTK_TEXT_VIEW (view));
 
 	check_spell_action = g_action_map_lookup_action (G_ACTION_MAP (priv->window),
 	                                                 "check-spell");
 	g_simple_action_set_enabled (G_SIMPLE_ACTION (check_spell_action),
-	                             (view != NULL) &&
-	                             gtk_text_view_get_editable (GTK_TEXT_VIEW (view)));
+				     editable_view);
 
 	config_spell_action = g_action_map_lookup_action (G_ACTION_MAP (priv->window),
 	                                                  "config-spell");
 	g_simple_action_set_enabled (G_SIMPLE_ACTION (config_spell_action),
-	                             (view != NULL) &&
-	                             gtk_text_view_get_editable (GTK_TEXT_VIEW (view)));
+				     editable_view);
 
 	inline_checker_action = g_action_map_lookup_action (G_ACTION_MAP (priv->window),
-							    "inline-checker");
+							    "inline-spell-checker");
 	g_simple_action_set_enabled (G_SIMPLE_ACTION (inline_checker_action),
-	                             (view != NULL) &&
-	                             gtk_text_view_get_editable (GTK_TEXT_VIEW (view)));
+				     editable_view);
 
-	if (view != NULL)
+	/* Update only on normal state to avoid garbage changes during e.g. file
+	 * loading.
+	 */
+	if (tab != NULL &&
+	    gedit_tab_get_state (tab) == GEDIT_TAB_STATE_NORMAL)
 	{
-		GeditTab *tab;
-		GtkTextBuffer *buffer;
+		GspellInlineCheckerText *inline_checker;
+		gboolean inline_checker_enabled;
 
-		tab = gedit_window_get_active_tab (priv->window);
-		g_return_if_fail (gedit_tab_get_view (tab) == view);
+		inline_checker = gspell_text_view_get_inline_checker (GTK_TEXT_VIEW (view));
+		inline_checker_enabled = gspell_inline_checker_text_get_enabled (inline_checker);
 
-		/* If the document is loading we can't get the metadata so we
-		 * endup with a useless speller.
-		 */
-		if (gedit_tab_get_state (tab) == GEDIT_TAB_STATE_NORMAL)
-		{
-			ViewData *data;
-			gboolean inline_checker_enabled;
-
-			data = g_object_get_data (G_OBJECT (view), VIEW_DATA_KEY);
-			inline_checker_enabled = data != NULL && data->inline_checker != NULL;
-
-			g_action_change_state (inline_checker_action,
-					       g_variant_new_boolean (inline_checker_enabled));
-		}
-
-		buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
-
-		g_simple_action_set_enabled (G_SIMPLE_ACTION (check_spell_action),
-		                             gtk_text_buffer_get_char_count (buffer) > 0);
+		g_action_change_state (inline_checker_action,
+				       g_variant_new_boolean (inline_checker_enabled));
 	}
 }
 
 static void
-set_inline_checker_from_metadata (ViewData *data)
+setup_inline_checker_from_metadata (GeditSpellPlugin *plugin,
+				    GeditView        *view)
 {
-	GeditSpellPlugin *plugin = data->plugin;
-	gboolean active = FALSE;
-	gchar *active_str;
+	GeditDocument *doc;
+	gboolean enabled = FALSE;
+	gchar *enabled_str;
+	GspellInlineCheckerText *inline_checker;
 	GeditView *active_view;
 
-	active_str = gedit_document_get_metadata (data->doc, GEDIT_METADATA_ATTRIBUTE_SPELL_ENABLED);
-	if (active_str != NULL)
+	doc = GEDIT_DOCUMENT (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
+
+	enabled_str = gedit_document_get_metadata (doc, GEDIT_METADATA_ATTRIBUTE_SPELL_ENABLED);
+	if (enabled_str != NULL)
 	{
-		active = g_str_equal (active_str, SPELL_ENABLED_STR);
-		g_free (active_str);
+		enabled = g_str_equal (enabled_str, SPELL_ENABLED_STR);
+		g_free (enabled_str);
 	}
 
-	set_inline_checker (data, active);
+	inline_checker = gspell_text_view_get_inline_checker (GTK_TEXT_VIEW (view));
+	gspell_inline_checker_text_set_enabled (inline_checker, enabled);
 
 	/* In case that the view is the active one we mark the spell action */
 	active_view = gedit_window_get_active_view (plugin->priv->window);
 
-	if (active_view == data->view)
+	if (active_view == view)
 	{
 		GAction *action;
 
 		action = g_action_map_lookup_action (G_ACTION_MAP (plugin->priv->window),
-		                                     "inline-checker");
-		g_action_change_state (action, g_variant_new_boolean (active));
+		                                     "inline-spell-checker");
+		g_action_change_state (action, g_variant_new_boolean (enabled));
 	}
 }
 
 static void
-on_document_loaded (GeditDocument *doc,
-		    ViewData      *data)
+language_notify_cb (GspellChecker *checker,
+		    GParamSpec    *pspec,
+		    GeditDocument *doc)
+{
+	const GspellLanguage *lang;
+	const gchar *language_code;
+
+	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
+
+	lang = gspell_checker_get_language (checker);
+	g_return_if_fail (lang != NULL);
+
+	language_code = gspell_language_get_code (lang);
+	g_return_if_fail (language_code != NULL);
+
+	gedit_document_set_metadata (doc,
+				     GEDIT_METADATA_ATTRIBUTE_SPELL_LANGUAGE, language_code,
+				     NULL);
+}
+
+static void
+on_document_loaded (GeditDocument    *doc,
+		    GeditSpellPlugin *plugin)
 {
 	GspellChecker *checker;
+	GeditTab *tab;
+	GeditView *view;
 
-	checker = GSPELL_CHECKER (g_object_get_qdata (G_OBJECT (doc), spell_checker_id));
+	checker = gspell_text_buffer_get_spell_checker (GTK_TEXT_BUFFER (doc));
 
 	if (checker != NULL)
 	{
@@ -615,48 +453,126 @@ on_document_loaded (GeditDocument *doc,
 		}
 	}
 
-	set_inline_checker_from_metadata (data);
+	tab = gedit_tab_get_from_document (doc);
+	view = gedit_tab_get_view (tab);
+	setup_inline_checker_from_metadata (plugin, view);
 }
 
 static void
 on_document_saved (GeditDocument *doc,
-		   ViewData      *data)
+		   gpointer       user_data)
 {
+	GeditTab *tab;
+	GeditView *view;
 	GspellChecker *checker;
 	const gchar *language_code = NULL;
+	GspellInlineCheckerText *inline_checker;
+	gboolean inline_checker_enabled;
 
 	/* Make sure to save the metadata here too */
 
-	checker = GSPELL_CHECKER (g_object_get_qdata (G_OBJECT (doc), spell_checker_id));
+	checker = gspell_text_buffer_get_spell_checker (GTK_TEXT_BUFFER (doc));
 
 	if (checker != NULL)
 	{
 		language_code = gspell_language_get_code (gspell_checker_get_language (checker));
 	}
 
+	tab = gedit_tab_get_from_document (doc);
+	view = gedit_tab_get_view (tab);
+
+	inline_checker = gspell_text_view_get_inline_checker (GTK_TEXT_VIEW (view));
+	inline_checker_enabled = gspell_inline_checker_text_get_enabled (inline_checker);
+
 	gedit_document_set_metadata (doc,
 	                             GEDIT_METADATA_ATTRIBUTE_SPELL_ENABLED,
-				     data->inline_checker != NULL ? SPELL_ENABLED_STR : NULL,
+				     inline_checker_enabled ? SPELL_ENABLED_STR : NULL,
 	                             GEDIT_METADATA_ATTRIBUTE_SPELL_LANGUAGE,
 	                             language_code,
 	                             NULL);
 }
 
+static void
+activate_spell_checking_in_view (GeditSpellPlugin *plugin,
+				 GeditView        *view)
+{
+	GeditDocument *doc;
+
+	doc = GEDIT_DOCUMENT (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
+
+	/* It is possible that a GspellChecker has already been set, for example
+	 * if a GeditTab has moved to another window.
+	 */
+	if (gspell_text_buffer_get_spell_checker (GTK_TEXT_BUFFER (doc)) == NULL)
+	{
+		const GspellLanguage *lang = get_language_from_metadata (doc);
+		GspellChecker *checker = gspell_checker_new (lang);
+
+		g_signal_connect_object (checker,
+					 "notify::language",
+					 G_CALLBACK (language_notify_cb),
+					 doc,
+					 0);
+
+		gspell_text_buffer_set_spell_checker (GTK_TEXT_BUFFER (doc), checker);
+		g_object_unref (checker);
+
+		setup_inline_checker_from_metadata (plugin, view);
+	}
+
+	g_signal_connect_object (doc,
+				 "loaded",
+				 G_CALLBACK (on_document_loaded),
+				 plugin,
+				 0);
+
+	g_signal_connect_object (doc,
+				 "saved",
+				 G_CALLBACK (on_document_saved),
+				 plugin,
+				 0);
+}
+
+static void
+disconnect_view (GeditSpellPlugin *plugin,
+		 GeditView        *view)
+{
+	GtkTextBuffer *buffer;
+
+	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
+
+	/* It should still be the same buffer as the one where the signal
+	 * handlers were connected. If not, we assume that the old buffer is
+	 * finalized. And it is anyway safe to call
+	 * g_signal_handlers_disconnect_by_func() if no signal handlers are
+	 * found.
+	 */
+	g_signal_handlers_disconnect_by_func (buffer, on_document_loaded, plugin);
+	g_signal_handlers_disconnect_by_func (buffer, on_document_saved, plugin);
+}
+
+static void
+deactivate_spell_checking_in_view (GeditSpellPlugin *plugin,
+				   GeditView        *view)
+{
+	GtkTextBuffer *buffer;
+	GspellInlineCheckerText *inline_checker;
+
+	disconnect_view (plugin, view);
+
+	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
+	gspell_text_buffer_set_spell_checker (buffer, NULL);
+
+	inline_checker = gspell_text_view_get_inline_checker (GTK_TEXT_VIEW (view));
+	gspell_inline_checker_text_set_enabled (inline_checker, FALSE);
+}
+
 static void
 tab_added_cb (GeditWindow      *window,
 	      GeditTab         *tab,
 	      GeditSpellPlugin *plugin)
 {
-	GeditView *view;
-	ViewData *data;
-
-	view = gedit_tab_get_view (tab);
-
-	data = view_data_new (plugin, view);
-	g_object_set_data_full (G_OBJECT (view),
-				VIEW_DATA_KEY,
-				data,
-				(GDestroyNotify) view_data_free);
+	activate_spell_checking_in_view (plugin, gedit_tab_get_view (tab));
 }
 
 static void
@@ -664,8 +580,12 @@ tab_removed_cb (GeditWindow      *window,
 		GeditTab         *tab,
 		GeditSpellPlugin *plugin)
 {
-	GeditView *view = gedit_tab_get_view (tab);
-	g_object_set_data (G_OBJECT (view), VIEW_DATA_KEY, NULL);
+	/* Don't deactivate completely the spell checking in @tab, since the tab
+	 * can be moved to another window and we don't want to loose the spell
+	 * checking settings (they are not saved in metadata for unsaved
+	 * documents).
+	 */
+	disconnect_view (plugin, gedit_tab_get_view (tab));
 }
 
 static void
@@ -673,8 +593,19 @@ gedit_spell_plugin_activate (GeditWindowActivatable *activatable)
 {
 	GeditSpellPlugin *plugin;
 	GeditSpellPluginPrivate *priv;
-	GtkStatusbar *statusbar;
-	GList *views, *l;
+	GList *views;
+	GList *l;
+
+	const GActionEntry action_entries[] =
+	{
+		{ "check-spell", check_spell_cb },
+		{ "config-spell", set_language_cb },
+		{ "inline-spell-checker",
+		  inline_checker_activate_cb,
+		  NULL,
+		  "false",
+		  inline_checker_change_state_cb }
+	};
 
 	gedit_debug (DEBUG_PLUGINS);
 
@@ -686,52 +617,50 @@ gedit_spell_plugin_activate (GeditWindowActivatable *activatable)
 	                                 G_N_ELEMENTS (action_entries),
 	                                 activatable);
 
-	statusbar = GTK_STATUSBAR (gedit_window_get_statusbar (priv->window));
-	priv->statusbar_context_id = gtk_statusbar_get_context_id (statusbar, "spell_plugin_message");
-
 	update_ui (plugin);
 
 	views = gedit_window_get_views (priv->window);
-	for (l = views; l != NULL; l = g_list_next (l))
+	for (l = views; l != NULL; l = l->next)
 	{
-		GeditView *view = GEDIT_VIEW (l->data);
-		ViewData *data = view_data_new (plugin, view);
-
-		g_object_set_data_full (G_OBJECT (view),
-					VIEW_DATA_KEY,
-					data,
-					(GDestroyNotify) view_data_free);
+		activate_spell_checking_in_view (plugin, GEDIT_VIEW (l->data));
 	}
 
-	priv->tab_added_id = g_signal_connect (priv->window,
-					       "tab-added",
-					       G_CALLBACK (tab_added_cb),
-					       activatable);
+	g_signal_connect (priv->window,
+			  "tab-added",
+			  G_CALLBACK (tab_added_cb),
+			  activatable);
 
-	priv->tab_removed_id = g_signal_connect (priv->window,
-						 "tab-removed",
-						 G_CALLBACK (tab_removed_cb),
-						 activatable);
+	g_signal_connect (priv->window,
+			  "tab-removed",
+			  G_CALLBACK (tab_removed_cb),
+			  activatable);
 }
 
 static void
 gedit_spell_plugin_deactivate (GeditWindowActivatable *activatable)
 {
+	GeditSpellPlugin *plugin;
 	GeditSpellPluginPrivate *priv;
+	GList *views;
+	GList *l;
 
 	gedit_debug (DEBUG_PLUGINS);
 
-	priv = GEDIT_SPELL_PLUGIN (activatable)->priv;
+	plugin = GEDIT_SPELL_PLUGIN (activatable);
+	priv = plugin->priv;
 
-	g_action_map_remove_action (G_ACTION_MAP (priv->window),
-	                            "check-spell");
-	g_action_map_remove_action (G_ACTION_MAP (priv->window),
-	                            "config-spell");
-	g_action_map_remove_action (G_ACTION_MAP (priv->window),
-	                            "inline-checker");
+	g_action_map_remove_action (G_ACTION_MAP (priv->window), "check-spell");
+	g_action_map_remove_action (G_ACTION_MAP (priv->window), "config-spell");
+	g_action_map_remove_action (G_ACTION_MAP (priv->window), "inline-spell-checker");
 
-	g_signal_handler_disconnect (priv->window, priv->tab_added_id);
-	g_signal_handler_disconnect (priv->window, priv->tab_removed_id);
+	g_signal_handlers_disconnect_by_func (priv->window, tab_added_cb, activatable);
+	g_signal_handlers_disconnect_by_func (priv->window, tab_removed_cb, activatable);
+
+	views = gedit_window_get_views (priv->window);
+	for (l = views; l != NULL; l = l->next)
+	{
+		deactivate_spell_checking_in_view (plugin, GEDIT_VIEW (l->data));
+	}
 }
 
 static void
@@ -742,28 +671,6 @@ gedit_spell_plugin_update_state (GeditWindowActivatable *activatable)
 	update_ui (GEDIT_SPELL_PLUGIN (activatable));
 }
 
-static void
-gedit_spell_plugin_class_init (GeditSpellPluginClass *klass)
-{
-	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-
-	object_class->dispose = gedit_spell_plugin_dispose;
-	object_class->set_property = gedit_spell_plugin_set_property;
-	object_class->get_property = gedit_spell_plugin_get_property;
-
-	if (spell_checker_id == 0)
-	{
-		spell_checker_id = g_quark_from_string ("GeditSpellCheckerID");
-	}
-
-	g_object_class_override_property (object_class, PROP_WINDOW, "window");
-}
-
-static void
-gedit_spell_plugin_class_finalize (GeditSpellPluginClass *klass)
-{
-}
-
 static void
 gedit_window_activatable_iface_init (GeditWindowActivatableInterface *iface)
 {
